/*
 * pages.h
 *
 * Copyright (c) Chris Putnam 2016-2019
 *
 * Program and source code released under the GPL version 2
 */
#ifndef PAGES_H
#define PAGES_H

#include "bibl.h"

int pages_add( fields *bibout, char *outtag, str *invalue, int level );

#endif

