/*
 * notes.h
 *
 * Copyright (c) Chris Putnam 2016-2019
 *
 * Source code released under the GPL version 2
 *
 */
#ifndef NOTES_H
#define NOTES_H

#include "bibutils.h"

int notes_add( fields *bibout, str *invalue, int level );

#endif

