/*
 * bibprogs.h
 *
 * Copyright (c) Chris Putnam 2008-2020
 * Copyright (c) Georgi N. Boshnakov 2020
 *
 * Program and source code released under the GPL version 2
 *
 */
#ifndef BIBPROGS_H
#define BIBPROGS_H

#include "bibutils.h"

// Georgi
//void bibprog( int argc, char *argv[], param *p, char *outfile[] );
double bibprog( int argc, char *argv[], param *p, char *outfile[] );

#endif
